
% Author: Sean Fulop 
% version September 2010
% The core WVD implementation comes from the Time-Frequency Toolbox of F. Auger et al.,
%

function [wvd, f, t] = WinWignerVille(signal, Fs, step, fftn, low, high, clip)
  if nargin ~= 7
    error('requires 7 arguments ')
  end

  % Arguments: signal is the signal (analytic only, please), with sampling rate Fs;
  % step is the "hop size" separating points of analysis
  % fftn is the transform analysis length, in samples;
  % low and high are the frequency limits of the display, in Hz;
  % clip is a negative number which specifies the number of dB down from
  % the maximum that are to be shown---everything quieter is clipped


len = length(signal);

t = 1:step:len;
% step size effectively introduces a time window, though it is not used for
% smoothing as in a spectrogram so there is no point in tapering.
% Larger steps assume longer-term stationarity of the signal (slowly
% varying)

N = fftn;

tcol = length(t);

wvd = zeros (N, tcol); %initialized Wigner-Ville distribution matrix

for icol = 1:tcol,
    ti = t(icol); %step through time indices where analysis will be performed
    taumax = min([ti-1, len-ti, round(N/2)-1]); %check to keep lag points within signal
    tau = -taumax:taumax; %set up lag window for the (discrete) distribution
    taulen = length(tau);
    tauwin = tftb_window(taulen,'Kaiser');
    indices = rem(N+tau, N)+1; %figure discrete frequencies to match up with the lag vector
    wvd(indices,icol) = tauwin .* signal(ti+tau) .* conj(signal(ti-tau)); 
    % this is the actual core of the transform, to be submitted to
    % a DFT later on; only one time slice of this "bilinear kernel"
    % or "instantaneous autocorrelation"
    % is computed here, and the for loop builds up the matrix
    % In this "windowed" version the lag window is multiplied by a tapering
    % function

    
    %    tau = round(N/2);
    
%    if (ti <= len-tau) & (ti >= tau+1),
%        wvd(tau+1, icol) = 0.5 * (signal(ti+tau) * conj(signal(ti-tau)) + ...
%            signal(ti-tau) * conj(signal(ti+tau)));
%    end;
end

wvd = fft(wvd); %each column of the resulting matrix is a time slice of the WVD
wvd = real(wvd);
% these steps complete the computation of the distribution

lowindex = 2*round(low/Fs*fftn);
if lowindex == 0
    lowindex = 1;
end
highindex = 2*round(high/Fs*fftn);
% user-input frequency bounds have to be doubled to work with the doubled 
% frequencies resulting from the WVD calculation method

wvd = wvd(lowindex:highindex, :); % truncate wvd to frequency range for plot
%WVDmag = abs(wvd); 
WVD = wvd ./ max(max(wvd)); % normalize so max magnitude will be 0 db
WVD = max(WVD, 0); % remove negative numbers by changing to zero
WVD = 20*log10(WVD); % convert to log magnitude, decibel scale


            
f = 0.5*Fs*[(lowindex-1):(highindex-1)] ./ fftn; 
%Wigner-Ville calculation has the effect of doubling the frequencies, hence
%the multiplier 0.5



 %clip tfd plot
WVDmag = max(WVD, clip);

%end

tforplot = t ./ Fs;

figure('Position',[400,300,900,400]);

pcolor(1000*tforplot, f, WVDmag );
axis xy;
colormap(flipud(gray));
shading interp;
xlabel('Time (ms)','FontName','times','FontSize',14)
ylabel('Frequency (Hz)','FontName','times','FontSize',14)
set(gca,'FontName','FixedWidth','FontSize',12)

%xlabel('Time (ms)','FontName','times','FontSize',12)
%ylabel('Frequency (Hz)','FontName','times','FontSize',12)


end
