
% Author: Sean Fulop 
% version September 2010
% The ZAM implementation comes from the Time-Frequency Toolbox of F. Auger et al.,
% protected by Gnu public license.

function [zam, f, t] = ZAMgray(signal, Fs, window, step, fwin, fftn, low, high, clip)
  if nargin ~= 9
    error('requires 9 arguments ')
  end

  % Arguments: signal is the signal (analytic only, please), with sampling rate Fs;
  % window is the length of the time window (must be an odd number of samples)
  % step is the "hop size" separating points of analysis
  % fwin is the size of frequency smoothing window (odd number of bins)
  % fftn is the transform analysis length, in samples;
  % low and high are the frequency limits of the display, in Hz;
  % clip is a negative number which specifies the number of dB down from
  % the maximum that are to be shown---everything quieter is clipped


len = length(signal);

t = 1:step:len;
N = fftn;

%hlength=floor(N/4); hlength=hlength+1-rem(hlength,2); 
h = tftb_window(fwin,'Kaiser');

[hrow,hcol]=size(h); Lh = (hrow-1)/2; h=h/h(Lh+1);
% Lh set equal to midpoint of smoothing window
% frequency window is normalized in amplitude

g = tftb_window(window,'Kaiser');
[grow,gcol]=size(g); Lg=(grow-1)/2; g=g/g(Lg+1);
if (gcol~=1)|(rem(grow,2)==0),
  error('time window must have odd length'); 
end;
% Lg set equal to midpoint of the smoothing window
% time window is normalized in amplitude

tcol = length(t);

zam = zeros (N, tcol); %initialized time-frequency distribution matrix

for icol = 1:tcol,
    ti = t(icol); %step through time indices where analysis will be performed
    taumax = min([ti+Lg-1, len-ti+Lg, round(N/2)-1,Lh]); %check to keep lag points within signal
    zam(1,icol) = g(Lg+1) * signal(ti) .* conj(signal(ti)); %set values in lowest bin
    
    for tau = 1:taumax,
        points = -min([tau,Lg,len-ti-tau]):min([tau,Lg,ti-tau-1]);
        g2 = g(Lg+1+points); %define lag window from time window
        R = sum(g2 .* signal(ti+tau-points) .* conj(signal(ti-tau-points)) );
        % R is the instantaneous autocorrelation (JeongWilliams1992,
        % Amin1987)
        zam( 1+tau, icol) = h(Lh+tau+1)*R;
        R = sum(g2 .* signal(ti-tau-points) .* conj(signal(ti+tau-points)) );
        zam( N+1-tau, icol) = h(Lh-tau+1)*R;
    end;
end;

zam = fft(zam); %each column of the resulting matrix is a time slice of the ZAMD
zam = real(zam);
% these steps complete the computation of the distribution

lowindex = 2*round(low/Fs*fftn);
if lowindex == 0
    lowindex = 1;
end
highindex = 2*round(high/Fs*fftn);
% user-input frequency bounds have to be doubled to work with the doubled 
% frequencies resulting from the TFD calculation method

zam = zam(lowindex:highindex, :); % truncate zamd to frequency range for plot
%ZAMmag = abs(zam); 
ZAM = zam ./ max(max(zam)); % normalize so max magnitude will be 0 db
ZAM = max(ZAM, 0); % remove negative numbers by changing to zero
ZAM = 20*log10(ZAM); % convert to log magnitude, decibel scale


            
f = 0.5*Fs*[(lowindex-1):(highindex-1)] ./ fftn; 
%General TFD calculation has the effect of doubling the frequencies, hence
%the multiplier 0.5



 %clip tfd plot
ZAMmag = max(ZAM, clip);

%end

tforplot = t ./ Fs;

figure('Position',[400,300,900,400]);

pcolor(1000*tforplot, f, ZAMmag );
axis xy;
colormap(flipud(gray));
%colormap(myjet);
shading interp;
xlabel('Time (ms)','FontName','times','FontSize',16)
ylabel('Frequency (Hz)','FontName','times','FontSize',16)
set(gca,'FontName','FixedWidth','FontSize',14)

%xlabel('Time (ms)','FontName','times','FontSize',12)
%ylabel('Frequency (Hz)','FontName','times','FontSize',12)


end
