% Revision 2009 author Sean Fulop
% This routine incorporates newly published method by O'Toole et al., see ref. [3]
% it is an adaptation of "hilbert.m" Copyright Mathworks

function x = newhilbert(xr,n)
%NEWHILBERT  Discrete-time analytic signal via Hilbert transform.
%   X = NEWHILBERT(Xr) computes the so-called discrete-time analytic signal
%   X = Xr + i*Xi such that Xi is the Hilbert transform of real vector Xr.
%   If the input Xr is complex, then only the real part is used: Xr=real(Xr).
%   If Xr is a matrix, then NEWHILBERT operates along the columns of Xr.
%
%   NEWHILBERT(Xr,N) computes the N-point Hilbert transform.  Xr is padded with 
%   zeros if it has less than N points, and truncated if it has more.  
%
%   For a discrete-time analytic signal X, the last half of fft(X) is zero, 
%   and the first (DC) and center (Nyquist) elements of fft(X) are purely real.
%
%   


%   References:
%     [1] Alan V. Oppenheim and Ronald W. Schafer, Discrete-Time
%     Signal Processing, 2nd ed., Prentice-Hall, Upper Saddle River, 
%     New Jersey, 1998.
%
%     [2] S. Lawrence Marple, Jr., Computing the discrete-time analytic 
%     signal via FFT, IEEE Transactions on Signal Processing, Vol. 47, 
%     No. 9, September 1999, pp.2600--2603.

%   [3] John O'Toole, Mostefa Mesbah and Boualem Boashash; A new discrete
%   analytic signal for reducing aliasing in the discrete Wigner-Ville
%   distribution; IEEE Trans. Sig. Proc. Vol 56(11) 2008.


if nargin<2, n=[]; end
if ~isreal(xr)
  warning('NEWHILBERT ignores imaginary part of input.')
  xr = real(xr);
end
zer = zeros(size(xr));
xr = [xr; zer];
% Work along the first nonsingleton dimension
[xr,nshifts] = shiftdim(xr);
if isempty(n)
  n = size(xr,1);
end
x = fft(xr,n,1); % n-point FFT over columns.
h  = zeros(n,~isempty(x)); % nx1 for nonempty. 0x0 for empty.
if n>0 && 2*fix(n/2)==n
  % even and nonempty
  h([1 n/2+1]) = 1;
  h(2:n/2) = 2;
elseif n>0
  % odd and nonempty
  h(1) = 1;
  h(2:(n+1)/2) = 2;
end
x = ifft(x.*h(:,ones(1,size(x,2))));
x = x(1:n/2);
% Convert back to the original shape.
x = shiftdim(x,-nshifts);